@@ -16,7 +16,7 @@
  *
  *
  * IDENTIFICATION
- *		$PostgreSQL: pgsql/src/bin/pg_dump/pg_backup_tar.c,v 1.46.4.1 2005/01/25 22:44:47 tgl Exp $
+ *		$PostgreSQL: pgsql/src/bin/pg_dump/pg_backup_tar.c,v 1.46.4.4 2007/08/29 16:31:57 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -483,7 +483,7 @@ _tarReadRaw(ArchiveHandle *AH, void *buf, size_t len, TAR_MEMBER *th, FILE *fh)
 			used = avail;
 
 		/* Copy, and adjust buffer pos */
-		memcpy(buf, AH->lookahead, used);
+		memcpy(buf, AH->lookahead + AH->lookaheadPos, used);
 		AH->lookaheadPos += used;
 
 		/* Adjust required length */
@@ -727,12 +727,13 @@ static int
 _ReadByte(ArchiveHandle *AH)
 {
 	lclContext *ctx = (lclContext *) AH->formatData;
-	int			res;
-	char		c = '\0';
+	size_t		res;
+	unsigned char c;
 
 	res = tarRead(&c, 1, ctx->FH);
-	if (res != EOF)
-		ctx->filePos += res;
+	if (res != 1)
+		die_horribly(AH, modulename, "unexpected end of file\n");
+	ctx->filePos += 1;
 	return c;
 }
 
@@ -1019,19 +1020,20 @@ _tarAddFile(ArchiveHandle *AH, TAR_MEMBER *th)
 	 */
 	fseeko(tmp, 0, SEEK_END);
 	th->fileLen = ftello(tmp);
+	fseeko(tmp, 0, SEEK_SET);
+
 	/*
 	 *	Some compilers with throw a warning knowing this test can never be
 	 *	true because off_t can't exceed the compared maximum.
 	 */
 	if (th->fileLen > MAX_TAR_MEMBER_FILELEN)
 		die_horribly(AH, modulename, "archive member too large for tar format\n");
-	fseeko(tmp, 0, SEEK_SET);
 
 	_tarWriteHeader(th);
 
-	while ((cnt = fread(&buf[0], 1, 32767, tmp)) > 0)
+	while ((cnt = fread(buf, 1, sizeof(buf), tmp)) > 0)
 	{
-		res = fwrite(&buf[0], 1, cnt, th->tarFH);
+		res = fwrite(buf, 1, cnt, th->tarFH);
 		if (res != cnt)
 			die_horribly(AH, modulename,
 						 "write error appending to tar archive (wrote %lu, attempted %lu)\n",
@@ -1040,15 +1042,16 @@ _tarAddFile(ArchiveHandle *AH, TAR_MEMBER *th)
 	}
 
 	if (fclose(tmp) != 0)		/* This *should* delete it... */
-		die_horribly(AH, modulename, "could not close tar member: %s\n", strerror(errno));
+		die_horribly(AH, modulename, "could not close tar member: %s\n",
+					 strerror(errno));
 
 	if (len != th->fileLen)
 	{
-		char		buf1[100],
-					buf2[100];
+		char		buf1[32],
+					buf2[32];
 
 		snprintf(buf1, sizeof(buf1), INT64_FORMAT, (int64) len);
-		snprintf(buf2, sizeof(buf2), INT64_FORMAT, (int64) th->pos);
+		snprintf(buf2, sizeof(buf2), INT64_FORMAT, (int64) th->fileLen);
 		die_horribly(AH, modulename, "actual file length (%s) does not match expected (%s)\n",
 					 buf1, buf2);
 	}
@@ -1155,7 +1158,6 @@ _tarGetHeader(ArchiveHandle *AH, TAR_MEMBER *th)
 	size_t		len;
 	unsigned long ullen;
 	off_t		hPos;
-	int			i;
 	bool		gotBlock = false;
 
 	while (!gotBlock)
@@ -1178,7 +1180,7 @@ _tarGetHeader(ArchiveHandle *AH, TAR_MEMBER *th)
 		hPos = ctx->tarFHpos;
 
 		/* Read a 512 byte block, return EOF, exit if short */
-		len = _tarReadRaw(AH, &h[0], 512, NULL, ctx->tarFH);
+		len = _tarReadRaw(AH, h, 512, NULL, ctx->tarFH);
 		if (len == 0)			/* EOF */
 			return 0;
 
@@ -1188,20 +1190,22 @@ _tarGetHeader(ArchiveHandle *AH, TAR_MEMBER *th)
 						 (unsigned long) len);
 
 		/* Calc checksum */
-		chk = _tarChecksum(&h[0]);
+		chk = _tarChecksum(h);
+		sscanf(&h[148], "%8o", &sum);
 
 		/*
-		 * If the checksum failed, see if it is a null block. If so, then
-		 * just try with next block...
+		 * If the checksum failed, see if it is a null block. If so,
+		 * silently continue to the next block.
 		 */
-
 		if (chk == sum)
 			gotBlock = true;
 		else
 		{
+			int			i;
+
 			for (i = 0; i < 512; i++)
 			{
-				if (h[0] != 0)
+				if (h[i] != 0)
 				{
 					gotBlock = true;
 					break;
@@ -1213,7 +1217,6 @@ _tarGetHeader(ArchiveHandle *AH, TAR_MEMBER *th)
 	sscanf(&h[0], "%99s", tag);
 	sscanf(&h[124], "%12lo", &ullen);
 	len = (size_t) ullen;
-	sscanf(&h[148], "%8o", &sum);
 
 	{
 		char		buf[100];
