@@ -4,10 +4,10 @@
  *	  definitions for executor state nodes
  *
  *
- * Portions Copyright (c) 1996-2003, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2005, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
- * $Id: execnodes.h,v 1.107.2.3 2007/04/26 23:25:40 tgl Exp $
+ * $PostgreSQL: pgsql/src/include/nodes/execnodes.h,v 1.122.4.1 2007/04/26 23:25:30 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -113,6 +113,10 @@ typedef struct ExprContext
 	Datum	   *ecxt_aggvalues; /* precomputed values for Aggref nodes */
 	bool	   *ecxt_aggnulls;	/* null flags for Aggref nodes */
 
+	/* Value to substitute for CaseTestExpr nodes in expression */
+	Datum		caseValue_datum;
+	bool		caseValue_isNull;
+
 	/* Value to substitute for CoerceToDomainValue nodes in expression */
 	Datum		domainValue_datum;
 	bool		domainValue_isNull;
@@ -205,7 +209,7 @@ typedef struct ProjectionInfo
  *	  This class is used to store information regarding junk attributes.
  *	  A junk attribute is an attribute in a tuple that is needed only for
  *	  storing intermediate information in the executor, and does not belong
- *	  in emitted tuples.	For example, when we do an UPDATE query,
+ *	  in emitted tuples.  For example, when we do an UPDATE query,
  *	  the planner adds a "junk" entry to the targetlist so that the tuples
  *	  returned to ExecutePlan() contain an extra attribute: the ctid of
  *	  the tuple to be updated.	This is needed to do the update, but we
@@ -214,12 +218,7 @@ typedef struct ProjectionInfo
  *	  real output tuple.
  *
  *	  targetList:		the original target list (including junk attributes).
- *	  length:			the length of 'targetList'.
- *	  tupType:			the tuple descriptor for the "original" tuple
- *						(including the junk attributes).
- *	  cleanTargetList:	the "clean" target list (junk attributes removed).
- *	  cleanLength:		the length of 'cleanTargetList'
- *	  cleanTupType:		the tuple descriptor of the "clean" tuple (with
+ *	  cleanTupType:		the tuple descriptor for the "clean" tuple (with
  *						junk attributes removed).
  *	  cleanMap:			A map with the correspondence between the non-junk
  *						attribute numbers of the "original" tuple and the
@@ -231,10 +230,6 @@ typedef struct JunkFilter
 {
 	NodeTag		type;
 	List	   *jf_targetList;
-	int			jf_length;
-	TupleDesc	jf_tupType;
-	List	   *jf_cleanTargetList;
-	int			jf_cleanLength;
 	TupleDesc	jf_cleanTupType;
 	AttrNumber *jf_cleanMap;
 	TupleTableSlot *jf_resultSlot;
@@ -286,7 +281,7 @@ typedef struct EState
 	/* Basic state for all query types: */
 	ScanDirection es_direction; /* current scan direction */
 	Snapshot	es_snapshot;	/* time qual to use */
-	Snapshot	es_crosscheck_snapshot;	/* crosscheck time qual for RI */
+	Snapshot	es_crosscheck_snapshot; /* crosscheck time qual for RI */
 	List	   *es_range_table; /* List of RangeTableEntrys */
 
 	/* Info about target table for insert/update/delete queries: */
@@ -311,7 +306,7 @@ typedef struct EState
 	List	   *es_rowMark;		/* not good place, but there is no other */
 
 	bool		es_instrument;	/* true requests runtime instrumentation */
-	bool		es_select_into;	/* true if doing SELECT INTO */
+	bool		es_select_into; /* true if doing SELECT INTO */
 	bool		es_into_oids;	/* true to generate OIDs in SELECT INTO */
 
 	List	   *es_exprcontexts;	/* List of ExprContexts within EState */
@@ -402,13 +397,25 @@ typedef HASH_SEQ_STATUS TupleHashIterator;
  *
  * It can also be instantiated directly for leaf Expr nodes that need no
  * local run-time state (such as Var, Const, or Param).
+ *
+ * To save on dispatch overhead, each ExprState node contains a function
+ * pointer to the routine to execute to evaluate the node.
  * ----------------
  */
-typedef struct ExprState
+
+typedef struct ExprState ExprState;
+
+typedef Datum (*ExprStateEvalFunc) (ExprState *expression,
+												ExprContext *econtext,
+												bool *isNull,
+												ExprDoneCond *isDone);
+
+struct ExprState
 {
 	NodeTag		type;
 	Expr	   *expr;			/* associated Expr node */
-} ExprState;
+	ExprStateEvalFunc evalfunc; /* routine to run to execute node */
+};
 
 /* ----------------
  *		GenericExprState node
@@ -494,8 +501,9 @@ typedef struct FuncExprState
 
 	/*
 	 * Flag to remember whether we have registered a shutdown callback for
-	 * this FuncExprState.  We do so only if setArgsValid has been true at
-	 * least once (since all the callback is for is to clear setArgsValid).
+	 * this FuncExprState.	We do so only if setArgsValid has been true at
+	 * least once (since all the callback is for is to clear
+	 * setArgsValid).
 	 */
 	bool		shutdown_reg;	/* a shutdown callback is registered */
 
@@ -559,6 +567,46 @@ typedef struct SubPlanState
 	FmgrInfo   *hashfunctions;	/* lookup data for hash functions */
 } SubPlanState;
 
+/* ----------------
+ *		FieldSelectState node
+ * ----------------
+ */
+typedef struct FieldSelectState
+{
+	ExprState	xprstate;
+	ExprState  *arg;			/* input expression */
+	TupleDesc	argdesc;		/* tupdesc for most recent input */
+} FieldSelectState;
+
+/* ----------------
+ *		FieldStoreState node
+ * ----------------
+ */
+typedef struct FieldStoreState
+{
+	ExprState	xprstate;
+	ExprState  *arg;			/* input tuple value */
+	List	   *newvals;		/* new value(s) for field(s) */
+	TupleDesc	argdesc;		/* tupdesc for most recent input */
+} FieldStoreState;
+
+/* ----------------
+ *		ConvertRowtypeExprState node
+ * ----------------
+ */
+typedef struct ConvertRowtypeExprState
+{
+	ExprState	xprstate;
+	ExprState  *arg;			/* input tuple value */
+	TupleDesc	indesc;			/* tupdesc for source rowtype */
+	TupleDesc	outdesc;		/* tupdesc for result rowtype */
+	AttrNumber *attrMap;		/* indexes of input fields, or 0 for null */
+	Datum	   *invalues;		/* workspace for deconstructing source */
+	char	   *innulls;
+	Datum	   *outvalues;		/* workspace for constructing result */
+	char	   *outnulls;
+} ConvertRowtypeExprState;
+
 /* ----------------
  *		CaseExprState node
  * ----------------
@@ -566,6 +614,7 @@ typedef struct SubPlanState
 typedef struct CaseExprState
 {
 	ExprState	xprstate;
+	ExprState  *arg;			/* implicit equality comparison argument */
 	List	   *args;			/* the arguments (list of WHEN clauses) */
 	ExprState  *defresult;		/* the default result (ELSE clause) */
 } CaseExprState;
@@ -597,6 +646,17 @@ typedef struct ArrayExprState
 	char		elemalign;		/* typalign of the element type */
 } ArrayExprState;
 
+/* ----------------
+ *		RowExprState node
+ * ----------------
+ */
+typedef struct RowExprState
+{
+	ExprState	xprstate;
+	List	   *args;			/* the arguments */
+	TupleDesc	tupdesc;		/* descriptor for result tuples */
+} RowExprState;
+
 /* ----------------
  *		CoalesceExprState node
  * ----------------
@@ -777,8 +837,10 @@ typedef ScanState SeqScanState;
 /* ----------------
  *	 IndexScanState information
  *
+ *		indxqualorig	   execution state for indxqualorig expressions
  *		NumIndices		   number of indices in this scan
  *		IndexPtr		   current index in use
+ *		MarkIndexPtr	   IndexPtr for marked scan point
  *		ScanKeys		   Skey structures to scan index rels
  *		NumScanKeys		   array of no of keys in each Skey struct
  *		RuntimeKeyInfo	   array of array of exprstates for Skeys
@@ -787,6 +849,7 @@ typedef ScanState SeqScanState;
  *		RuntimeKeysReady   true if runtime Skeys have been computed
  *		RelationDescs	   ptr to array of relation descriptors
  *		ScanDescs		   ptr to array of scan descriptors
+ *		LossyQuals		   ptr to array of qual lists for lossy operators
  *		DupHash			   hashtable for recognizing dups in multiple scan
  *		MaxHash			   max # entries we will allow in hashtable
  * ----------------
@@ -794,7 +857,6 @@ typedef ScanState SeqScanState;
 typedef struct IndexScanState
 {
 	ScanState	ss;				/* its first field is NodeTag */
-	List	   *indxqual;
 	List	   *indxqualorig;
 	int			iss_NumIndices;
 	int			iss_IndexPtr;
@@ -806,6 +868,7 @@ typedef struct IndexScanState
 	bool		iss_RuntimeKeysReady;
 	RelationPtr iss_RelationDescs;
 	IndexScanDescPtr iss_ScanDescs;
+	List	  **iss_LossyQuals;
 	HTAB	   *iss_DupHash;
 	long		iss_MaxHash;
 } IndexScanState;
