@@ -7,10 +7,10 @@
  * Client-side code should include postgres_fe.h instead.
  *
  *
- * Portions Copyright (c) 1996-2003, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2005, PostgreSQL Global Development Group
  * Portions Copyright (c) 1995, Regents of the University of California
  *
- * $Id: postgres.h,v 1.65.4.1 2008/03/25 19:31:53 tgl Exp $
+ * $PostgreSQL: pgsql/src/include/postgres.h,v 1.70.4.1 2008/03/25 19:31:46 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -59,8 +59,6 @@
  * TOASTed.
  * ----------------
  */
-#define TUPLE_TOASTER_ACTIVE
-
 typedef struct varattrib
 {
 	int32		va_header;		/* External/compressed storage */
@@ -550,9 +548,6 @@ extern int ExceptionalCondition(char *conditionName, char *errorType,
 #define DATA(x)   extern int no_such_variable
 #define DESCR(x)  extern int no_such_variable
 
-#define BKI_BEGIN
-#define BKI_END
-
 typedef int4 aclitem;			/* PHONY definition for catalog use only */
 
 #endif   /* POSTGRES_H */
