@@ -8,7 +8,7 @@
 #include "dt.h"
 #include "pgtypes_timestamp.h"
 
-static int	day_tab[2][13] = {
+int	day_tab[2][13] = {
 	{31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31, 0},
 {31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31, 0}};
 
@@ -1065,8 +1065,8 @@ abstime2tm(AbsoluteTime _time, int *tzp, struct tm * tm, char **tzn)
 			 * case it contains an error message, which doesn't fit in the
 			 * buffer
 			 */
-			StrNCpy(*tzn, tzname[tm->tm_isdst], MAXTZLEN + 1);
-			if (strlen(tzname[tm->tm_isdst]) > MAXTZLEN)
+			StrNCpy(*tzn, TZNAME_GLOBAL[tm->tm_isdst], MAXTZLEN + 1);
+			if (strlen(TZNAME_GLOBAL[tm->tm_isdst]) > MAXTZLEN)
 				tm->tm_isdst = -1;
 		}
 	}
@@ -1847,7 +1847,7 @@ ParseDateTime(char *timestr, char *lowstr,
 				{
 					ftype[nf] = DTK_DATE;
 					while (isalnum((unsigned char) *(*endstr)) || (*(*endstr) == *dp))
-						*lp++ = tolower((unsigned char) *(*endstr)++);
+						*lp++ = pg_tolower((unsigned char) *(*endstr)++);
 				}
 			}
 
@@ -1875,9 +1875,9 @@ ParseDateTime(char *timestr, char *lowstr,
 		else if (isalpha((unsigned char) *(*endstr)))
 		{
 			ftype[nf] = DTK_STRING;
-			*lp++ = tolower((unsigned char) *(*endstr)++);
+			*lp++ = pg_tolower((unsigned char) *(*endstr)++);
 			while (isalpha((unsigned char) *(*endstr)))
-				*lp++ = tolower((unsigned char) *(*endstr)++);
+				*lp++ = pg_tolower((unsigned char) *(*endstr)++);
 
 			/*
 			 * Full date string with leading text month? Could also be a
@@ -1919,9 +1919,9 @@ ParseDateTime(char *timestr, char *lowstr,
 			else if (isalpha((unsigned char) *(*endstr)))
 			{
 				ftype[nf] = DTK_SPECIAL;
-				*lp++ = tolower((unsigned char) *(*endstr)++);
+				*lp++ = pg_tolower((unsigned char) *(*endstr)++);
 				while (isalpha((unsigned char) *(*endstr)))
-					*lp++ = tolower((unsigned char) *(*endstr)++);
+					*lp++ = pg_tolower((unsigned char) *(*endstr)++);
 			}
 			/* otherwise something wrong... */
 			else
@@ -2763,7 +2763,7 @@ PGTYPEStimestamp_defmt_scan(char **str, char *fmt, timestamp *d,
 			}
 			else
 			{
-				/* XXX Error: no match */
+				/* Error: no match */
 				err = 1;
 				return err;
 			}
@@ -3115,7 +3115,7 @@ PGTYPEStimestamp_defmt_scan(char **str, char *fmt, timestamp *d,
 				 */
 				for (j = 0; !err && j < szdatetktbl; j++)
 				{
-					if (strcasecmp(datetktbl[j].token, scan_val.str_val) == 0)
+					if (pg_strcasecmp(datetktbl[j].token, scan_val.str_val) == 0)
 					{
 						/*
 						 * tz calculates the offset for the seconds, the
